@@ -2,10 +2,15 @@
 
 import argparse
 import math
+import os
+import shutil
+import tempfile
+import time
 
 import ezdxf
 
 from ..calc import calc_distance, point_of_line  # pylint: disable=E0402
+from ..dxfcolors import dxfcolors
 from ..input_plugins_base import DrawReaderBase
 from ..vc_types import VcSegment
 
@@ -18,14 +23,16 @@ try:
         from ezdxf.addons import text2path
 
         SUPPORT_TEXT = True
-    except Exception:  # pylint: disable=W0703
-        print("for text support, please install matplotlib")
+    except Exception as error:  # pylint: disable=W0703
+        print(f"WARNING: for text support, please install matplotlib: {error}")
         SUPPORT_TEXT = False
 
-except Exception:  # pylint: disable=W0703
-    print("WARNING: please install newer version of ezdxf")
+except Exception as error:  # pylint: disable=W0703
+    print(f"WARNING: please install newer version of ezdxf: {error}")
     SUPPORT_TEXT = False
 
+BITMAP_FORMATS = ("bmp", "png", "gif", "jpg", "jpeg")
+
 
 class DrawReader(DrawReaderBase):
 
@@ -34,6 +41,15 @@ class DrawReader(DrawReaderBase):
     MIN_DIST = 0.0001
 
     can_save_tabs = True
+    can_save_setup = True
+    can_load_setup = True
+    color_layers = False
+    select_layers = []
+    filtered_layers = []
+    selected_layers = []
+    cam_setup = ""
+
+    backup_ok = False
 
     @staticmethod
     def arg_parser(parser) -> None:
@@ -43,41 +59,178 @@ class DrawReader(DrawReaderBase):
             type=float,
             default=0.0,
         )
+        parser.add_argument(
+            "--dxfread-color-layers",
+            help="dxfread: using different colors as different layers",
+            type=bool,
+            default=False,
+        )
+        parser.add_argument(
+            "--dxfread-select-layers",
+            help="dxfread: selecting layers by name",
+            type=str,
+            default=[],
+            action="append",
+        )
+        if os.path.isfile("/usr/share/inkscape/extensions/dxf_outlines.py"):
+            parser.add_argument(
+                "--dxfread-no-svg",
+                help="dxfread: disable svg support (inkscape converter)",
+                action="store_true",
+            )
+        if os.path.isfile("/usr/bin/potrace"):
+            parser.add_argument(
+                "--dxfread-no-bmp",
+                help="dxfread: disable bmp support (potrace converter)",
+                action="store_true",
+            )
+
+    @staticmethod
+    def preload_setup(filename: str, args: argparse.Namespace):
+        from PyQt5.QtWidgets import (  # pylint: disable=E0611,C0415
+            QCheckBox,
+            QDialog,
+            QDialogButtonBox,
+            QDoubleSpinBox,
+            QLabel,
+            QVBoxLayout,
+        )
+
+        dialog = QDialog()
+        dialog.setWindowTitle("DXF-Reader")
+
+        dialog.buttonBox = QDialogButtonBox(QDialogButtonBox.Ok)
+        dialog.buttonBox.accepted.connect(dialog.accept)
+
+        dialog.layout = QVBoxLayout()
+        message = QLabel("Import-Options")
+        dialog.layout.addWidget(message)
+
+        dxfread_color_layers = QCheckBox("split colors into Layers ?")
+        dxfread_color_layers.setChecked(args.dxfread_color_layers)
+        dialog.layout.addWidget(dxfread_color_layers)
+
+        label = QLabel("Scale")
+        dialog.layout.addWidget(label)
+        dxfread_scale = QDoubleSpinBox()
+        dxfread_scale.setDecimals(4)
+        dxfread_scale.setSingleStep(0.1)
+        dxfread_scale.setMinimum(0.0001)
+        dxfread_scale.setMaximum(100000)
+        dxfread_scale.setValue(1.0)
+        dialog.layout.addWidget(dxfread_scale)
+
+        dxfread_select_layers = {}
+        if filename.lower().endswith(".dxf"):
+            doc = ezdxf.readfile(filename)
+            for layer in doc.layers:
+                dxfread_select_layers[layer.dxf.name] = QCheckBox(
+                    f"select layer: {layer.dxf.name}"
+                )
+                dxfread_select_layers[layer.dxf.name].setChecked(True)
+                dialog.layout.addWidget(dxfread_select_layers[layer.dxf.name])
+
+        dialog.layout.addWidget(dialog.buttonBox)
+        dialog.setLayout(dialog.layout)
+
+        if dialog.exec():
+            args.dxfread_color_layers = dxfread_color_layers.isChecked()
+            args.dxfread_scale = dxfread_scale.value()
+            if not args.dxfread_color_layers:
+                selection = []
+                for layer, stat in dxfread_select_layers.items():
+                    if stat.isChecked():
+                        selection.append(layer)
+                args.dxfread_select_layers = selection
 
     def __init__(
         self, filename: str, args: argparse.Namespace = None
     ):  # pylint: disable=W0613
         """converting dxf into single segments."""
         self.filename = filename
-        self.doc = ezdxf.readfile(self.filename)
+        if self.filename.lower().endswith(".svg") and os.path.isfile(
+            "/usr/share/inkscape/extensions/dxf_outlines.py"
+        ):
+            print("INFO: converting svg to dxf with inkscape")
+            print("    you can disable this with: --dxfread-no-svg")
+            _fd, tmp_path = tempfile.mkstemp()
+            os.system(
+                f"cd /usr/share/inkscape/extensions/ ; python3 dxf_outlines.py --output='{tmp_path}' '{os.path.realpath(self.filename)}'"
+            )
+            self.doc = ezdxf.readfile(tmp_path)
+            os.remove(tmp_path)
+        elif self.filename.lower().endswith(BITMAP_FORMATS) and os.path.isfile(
+            "/usr/bin/potrace"
+        ):
+            print("INFO: converting bitmap to dxf with potrace")
+            print("    you can disable this with: --dxfread-no-bmp")
+            _fd, tmp_path = tempfile.mkstemp()
+
+            if not self.filename.lower().endswith(".bmp"):
+                _fd2, tmp_path2 = tempfile.mkstemp()
+                os.system(
+                    f"convert '{os.path.realpath(self.filename)}' '{tmp_path2}.bmp'"
+                )
+                os.system(f"potrace -b dxf -o '{tmp_path}' '{tmp_path2}.bmp'")
+                os.remove(f"{tmp_path2}.bmp")
+            else:
+                os.system(
+                    f"potrace -b dxf -o '{tmp_path}' '{os.path.realpath(self.filename)}'"
+                )
+            self.doc = ezdxf.readfile(tmp_path)
+            os.remove(tmp_path)
+        else:
+            self.doc = ezdxf.readfile(self.filename)
+
         if args is None or args.dxfread_scale == 0.0:
             self.scale = 1.0
             try:
                 if self.doc.units != 0:
                     self.scale = ezdxf.units.conversion_factor(self.doc.units, ezdxf.units.MM)  # type: ignore
-            except Exception:  # pylint: disable=W0703
+            except Exception as error:  # pylint: disable=W0703,W0621
                 print("UNKNOWN UNITS")
-                print("WARNING: please install newer version of ezdxf")
+                print(f"WARNING: please install newer version of ezdxf: {error}")
         else:
             self.scale = args.dxfread_scale
 
+        if args is not None:
+            self.color_layers = args.dxfread_color_layers
+            self.select_layers = []
+            for layer_name in args.dxfread_select_layers:
+                if "," in layer_name:
+                    self.select_layers += layer_name.split(",")
+                else:
+                    self.select_layers.append(layer_name)
+
         self.segments: list[dict] = []
         self.model_space = self.doc.modelspace()
+        self.layer_colors = {}
+        for layer in self.doc.layers:
+            self.layer_colors[layer.dxf.name] = layer.dxf.color
 
         try:
             with MTextExplode(self.model_space) as xpl:
                 for mtext in self.model_space.query("MTEXT"):
-                    xpl.explode(mtext)
-        except Exception:  # pylint: disable=W0703
-            print("WARNING: please install newer version of ezdxf")
+                    if mtext.dxf.layer == "_CAMCFG":
+                        # read setup data from dxf
+                        self.cam_setup = mtext.text.replace("\\P", "\n")
+                    else:
+                        xpl.explode(mtext)
+        except Exception as error:  # pylint: disable=W0703
+            print(f"WARNING: can not explore MText: {error}")
 
-        for element in self.model_space:
+        part_l = len(self.model_space)
+        for part_n, element in enumerate(self.model_space):
+            print(f"loading file: {round((part_n + 1) * 100 / part_l, 1)}%", end="\r")
+            if element.dxf.layer == "_CAMCFG":
+                continue
             dxftype = element.dxftype()
             if dxftype in self.VTYPES:
                 for v_element in element.virtual_entities():  # type: ignore
                     self.add_entity(v_element)
             else:
                 self.add_entity(element)
+        print("")
 
         self.min_max = [0.0, 0.0, 10.0, 10.0]
         for seg_idx, segment in enumerate(self.segments):
@@ -97,8 +250,29 @@ class DrawReader(DrawReaderBase):
         self.size.append(self.min_max[2] - self.min_max[0])
         self.size.append(self.min_max[3] - self.min_max[1])
 
+        if self.filtered_layers:
+            print(f"dxfread: filtered layers: {', '.join(self.filtered_layers)}")
+        if self.selected_layers:
+            print(f"dxfread: selected layers: {', '.join(self.selected_layers)}")
+
     def add_entity(self, element, offset: tuple = (0, 0)):
         dxftype = element.dxftype()
+        layer = element.dxf.layer
+        color = element.dxf.color
+        if color == 256:
+            color = self.layer_colors[layer]
+
+        if self.color_layers:
+            colorname = dxfcolors[color][3] or f"c{color}"
+            layer = f"{layer}-{colorname}"
+
+        if self.select_layers and layer not in self.select_layers:
+            if layer not in self.filtered_layers:
+                self.filtered_layers.append(layer)
+            return
+        if layer not in self.selected_layers:
+            self.selected_layers.append(layer)
+
         if dxftype in self.VTYPES:
             for v_element in element.virtual_entities():  # type: ignore
                 self.add_entity(v_element)
@@ -111,7 +285,9 @@ class DrawReader(DrawReaderBase):
             text_offset = (offset[0] + pos[0], offset[1] + pos[1])
             text_offset = (offset[0] + pos[0], offset[1] + pos[1])
             for path in paths:
-                self._add_path(path, text_offset, pscale=scale, layer=element.dxf.layer)
+                self._add_path(
+                    path, text_offset, pscale=scale, layer=layer, color=color
+                )
 
         elif dxftype == "LINE":
             dist = calc_distance(
@@ -124,7 +300,8 @@ class DrawReader(DrawReaderBase):
                         {
                             "type": dxftype,
                             "object": None,
-                            "layer": element.dxf.layer,
+                            "layer": layer,
+                            "color": color,
                             "start": (
                                 (element.dxf.start.x + offset[0]) * self.scale,
                                 (element.dxf.start.y + offset[1]) * self.scale,
@@ -140,7 +317,7 @@ class DrawReader(DrawReaderBase):
 
         elif dxftype in self.PTYPES:
             path = make_path(element)
-            self._add_path(path, offset, layer=element.dxf.layer)
+            self._add_path(path, offset, layer=layer, color=color)
 
         elif dxftype in {"ARC", "CIRCLE"}:
             if dxftype == "CIRCLE":
@@ -174,7 +351,7 @@ class DrawReader(DrawReaderBase):
             if steps > 0:
                 astep = adiff / steps
                 angle = start_angle
-                for step_n in range(0, steps):  # pylint: disable=W0612
+                for _step_n in range(0, steps):  # pylint: disable=W0612
                     (start, end, bulge) = ezdxf.math.arc_to_bulge(
                         element.dxf.center,
                         angle / 180 * math.pi,
@@ -188,7 +365,8 @@ class DrawReader(DrawReaderBase):
                                 {
                                     "type": dxftype,
                                     "object": None,
-                                    "layer": element.dxf.layer,
+                                    "layer": layer,
+                                    "color": color,
                                     "start": (
                                         (start.x + offset[0]) * self.scale,
                                         (start.y + offset[1]) * self.scale,
@@ -223,7 +401,8 @@ class DrawReader(DrawReaderBase):
                             {
                                 "type": dxftype,
                                 "object": None,
-                                "layer": element.dxf.layer,
+                                "layer": layer,
+                                "color": color,
                                 "start": (
                                     (start.x + offset[0]) * self.scale,
                                     (start.y + offset[1]) * self.scale,
@@ -248,7 +427,7 @@ class DrawReader(DrawReaderBase):
             for attrib in element.dxf.__dict__:
                 print(f"  element.dxf.{attrib} = {getattr(element.dxf, attrib)}")
 
-    def _add_path(self, path, offset, pscale=1.0, layer="0") -> list[float]:
+    def _add_path(self, path, offset, pscale=1.0, layer="0", color=256) -> list[float]:
         last = path.start
         for command in path:
             if command.type == Command.LINE_TO:
@@ -261,6 +440,7 @@ class DrawReader(DrawReaderBase):
                                 "type": "LINE",
                                 "object": None,
                                 "layer": layer,
+                                "color": color,
                                 "start": (
                                     (last[0] * pscale + offset[0]) * self.scale,
                                     (last[1] * pscale + offset[1]) * self.scale,
@@ -305,6 +485,7 @@ class DrawReader(DrawReaderBase):
                                     "type": "LINE",
                                     "object": None,
                                     "layer": layer,
+                                    "color": color,
                                     "start": (
                                         (last[0] * pscale + offset[0]) * self.scale,
                                         (last[1] * pscale + offset[1]) * self.scale,
@@ -328,6 +509,7 @@ class DrawReader(DrawReaderBase):
                                 "type": "LINE",
                                 "object": None,
                                 "layer": layer,
+                                "color": color,
                                 "start": (
                                     (last[0] * pscale + offset[0]) * self.scale,
                                     (last[1] * pscale + offset[1]) * self.scale,
@@ -347,6 +529,15 @@ class DrawReader(DrawReaderBase):
         return last
 
     def save_tabs(self, tabs: list) -> None:
+
+        if not self.backup_ok:
+            try:
+                shutil.copy2(self.filename, f"{self.filename}.{int(time.time())}")
+                self.backup_ok = True
+            except Exception as error:  # pylint: disable=W0703,W0621
+                print(f"ERROR: can not make backup of file: {self.filename}: {error}")
+                return
+
         delete_layers = []
         for layer in self.doc.layers:
             if layer.dxf.name.startswith("BREAKS:") or layer.dxf.name.startswith(
@@ -372,6 +563,15 @@ class DrawReader(DrawReaderBase):
             )
 
     def save_starts(self, objects: dict) -> None:
+
+        if not self.backup_ok:
+            try:
+                shutil.copy2(self.filename, f"/{self.filename}.{int(time.time())}")
+                self.backup_ok = True
+            except Exception as error:  # pylint: disable=W0703,W0621
+                print(f"ERROR: can not make backup of file: {self.filename}: {error}")
+                return
+
         delete_layers = []
         for layer in self.doc.layers:
             if layer.dxf.name.startswith("_STARTS"):
@@ -397,6 +597,53 @@ class DrawReader(DrawReaderBase):
                 f"ERROR while saving tabs to dxf file ({self.filename}): {save_error}"
             )
 
+    def save_setup(self, setup: str) -> None:
+
+        if not self.backup_ok:
+            try:
+                shutil.copy2(self.filename, f"{self.filename}.{int(time.time())}")
+                self.backup_ok = True
+            except Exception as error:  # pylint: disable=W0703,W0621
+                print(f"ERROR: can not make backup of file: {self.filename}: {error}")
+                return
+
+        delete_layers = []
+        for layer in self.doc.layers:
+            if layer.dxf.name.startswith("_CAMCFG"):
+                delete_layers.append(layer.dxf.name)
+
+        for layer_name in delete_layers:
+            for element in self.model_space:
+                if element.dxf.layer == layer_name:
+                    element.destroy()
+            self.doc.layers.remove(layer_name)
+
+        tabs_layer = self.doc.layers.add("_CAMCFG")
+        tabs_layer.color = 1
+        self.model_space.add_mtext(
+            setup, dxfattribs={"style": "DejaVu Sans", "layer": "_CAMCFG"}
+        )
+        try:
+            self.doc.saveas(self.filename)
+            self.cam_setup = setup
+        except Exception as save_error:  # pylint: disable=W0703
+            print(
+                f"ERROR while saving setup to dxf file ({self.filename}): {save_error}"
+            )
+
     @staticmethod
-    def suffix() -> list[str]:
-        return ["dxf"]
+    def suffix(args: argparse.Namespace = None) -> list[str]:
+        suffixes = ["dxf"]
+        if not hasattr(args, "dxfread_no_bmp") or (
+            not args.dxfread_no_svg
+            and os.path.isfile("/usr/share/inkscape/extensions/dxf_outlines.py")
+        ):
+            suffixes.append("svg")
+        if not hasattr(args, "dxfread_no_bmp") or (
+            not args.dxfread_no_bmp and os.path.isfile("/usr/bin/potrace")
+        ):
+            if os.path.isfile("/usr/bin/convert"):
+                suffixes += BITMAP_FORMATS
+            else:
+                suffixes.append("bmp")
+        return suffixes
