@@ -18,6 +18,10 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
         if self.comments:
             self.gcode.append("")
 
+    def raw(self, cmd) -> None:
+        if cmd:
+            self.gcode.append(cmd)
+
     def g64(self, value) -> None:
         self.gcode.append(f"G64 P{value}")
 
@@ -73,7 +77,7 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
         if not soft and offsets != (0.0, 0.0, 0.0):
             self.offsets_reset = True
             self.gcode.append(
-                f"G10 L2 P1 X{offsets[0]} Y{offsets[1]} Z{offsets[2]} (workpiece offsets for G54)"
+                f"G10 L2 P1 X{offsets[0]:.6f} Y{offsets[1]:.6f} Z{offsets[2]:.6f} (workpiece offsets for G54)"
             )
             self.gcode.append("G54")
         if soft:
@@ -95,13 +99,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     def move(self, x_pos=None, y_pos=None, z_pos=None) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6):.6f}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6):.6f}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6):.6f}")
             self.z_pos = z_pos
         if line:
             self.gcode.append("G00 " + " ".join(line))
@@ -148,13 +152,13 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     def linear(self, x_pos=None, y_pos=None, z_pos=None) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6):.6f}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6):.6f}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6):.6f}")
             self.z_pos = z_pos
         if line:
             self.gcode.append("G01 " + " ".join(line))
@@ -164,20 +168,20 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     ) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6):.6f}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6):.6f}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6):.6f}")
             self.z_pos = z_pos
         if i_pos is not None:
-            line.append(f"I{round(i_pos, 5)}")
+            line.append(f"I{round(i_pos, 6):.6f}")
         if j_pos is not None:
-            line.append(f"J{round(j_pos, 5)}")
+            line.append(f"J{round(j_pos, 6):.6f}")
         if r_pos is not None:
-            line.append(f"R{round(r_pos, 5)}")
+            line.append(f"R{round(r_pos, 6):.6f}")
         if line:
             self.gcode.append("G02 " + " ".join(line))
 
@@ -186,25 +190,37 @@ class PostProcessorGcodeLinuxCNC(PostProcessor):
     ) -> None:
         line = []
         if x_pos is not None and self.x_pos != x_pos:
-            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6)}")
+            line.append(f"X{round((x_pos + self.offsets[0]) * self.scale, 6):.6f}")
             self.x_pos = x_pos
         if y_pos is not None and self.y_pos != y_pos:
-            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6)}")
+            line.append(f"Y{round((y_pos + self.offsets[1]) * self.scale, 6):.6f}")
             self.y_pos = y_pos
         if z_pos is not None and self.z_pos != z_pos:
-            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6)}")
+            line.append(f"Z{round((z_pos + self.offsets[2]) * self.scale, 6):.6f}")
             self.z_pos = z_pos
         if i_pos is not None:
-            line.append(f"I{round(i_pos, 5)}")
+            line.append(f"I{round(i_pos, 6):.6f}")
         if j_pos is not None:
-            line.append(f"J{round(j_pos, 5)}")
+            line.append(f"J{round(j_pos, 6):.6f}")
         if r_pos is not None:
-            line.append(f"R{round(r_pos, 5)}")
+            line.append(f"R{round(r_pos, 6):.6f}")
         if line:
             self.gcode.append("G03 " + " ".join(line))
 
-    def get(self) -> str:
-        return "\n".join(self.gcode)
+    def get(self, numbers=False) -> str:
+        output = []
+        if numbers:
+            line_number = 1
+            nlen = len(str(len(self.gcode))) + 1
+            for line in self.gcode:
+                if not line or line[0] in {" ", "("}:
+                    output.append(line)
+                else:
+                    output.append(f"N{str(line_number).zfill(nlen)} {line}")
+                    line_number += 1
+        else:
+            output = self.gcode
+        return "\n".join(output)
 
     @staticmethod
     def suffix() -> str:
