@@ -22,6 +22,10 @@ class PostProcessorHpgl(PostProcessor):
         if self.comments:
             self.hpgl.append("")
 
+    def raw(self, cmd) -> None:
+        if cmd:
+            self.hpgl.append(cmd)
+
     def absolute(self, active=True) -> None:
         if active:
             self.hpgl.append("PA")
@@ -165,7 +169,7 @@ class PostProcessorHpgl(PostProcessor):
             self.last_x = self.x_pos
             self.last_y = self.y_pos
 
-    def get(self) -> str:
+    def get(self, numbers=False) -> str:  # pylint: disable=W0613
         output = ""
         last_word = ""
         for cmd in self.hpgl:
