@@ -1,21 +1,12 @@
 def setup_defaults(_) -> dict:
     return {
         "mill": {
-            "fast_move_z": {
-                "default": 5.0,
-                "type": "float",
-                "min": 0.0,
-                "max": 999.0,
-                "title": _("Fast-Move Z"),
-                "tooltip": _("the Z-Position for fast moves"),
-            },
-            "G64": {
-                "default": 0.020000,
-                "type": "float",
-                "min": 0.0,
-                "max": 0.1,
-                "title": _("G64-Value"),
-                "tooltip": _("value for the G64 command"),
+            "active": {
+                "default": True,
+                "type": "bool",
+                "per_object": True,
+                "title": _("Active"),
+                "tooltip": _("enable/disable this object"),
             },
             "depth": {
                 "default": -9.0,
@@ -25,6 +16,7 @@ def setup_defaults(_) -> dict:
                 "per_object": True,
                 "title": _("Depth"),
                 "tooltip": _("the end depth for milling"),
+                "unit": "LINEARMEASURE",
             },
             "step": {
                 "default": -9.0,
@@ -34,13 +26,26 @@ def setup_defaults(_) -> dict:
                 "per_object": True,
                 "title": _("Step"),
                 "tooltip": _("the maximum depth in one move"),
+                "unit": "LINEARMEASURE",
             },
-            "active": {
-                "default": True,
-                "type": "bool",
+            "start_depth": {
+                "default": 0.0,
+                "type": "float",
+                "min": -999.0,
+                "max": 0.0,
                 "per_object": True,
-                "title": _("Active"),
-                "tooltip": _("enable/disable this object"),
+                "title": _("Start-Depth"),
+                "tooltip": _("the start depth for milling"),
+                "unit": "LINEARMEASURE",
+            },
+            "passes": {
+                "default": 1,
+                "type": "int",
+                "min": 1,
+                "max": 99,
+                "per_object": True,
+                "title": _("Passes"),
+                "tooltip": _("Passes / in Laser-Mode"),
             },
             "helix_mode": {
                 "default": False,
@@ -49,6 +54,23 @@ def setup_defaults(_) -> dict:
                 "title": _("Helix"),
                 "tooltip": _("Helix"),
             },
+            "fast_move_z": {
+                "default": 5.0,
+                "type": "float",
+                "min": 0.0,
+                "max": 999.0,
+                "title": _("Fast-Move Z"),
+                "tooltip": _("the Z-Position for fast moves"),
+                "unit": "LINEARMEASURE",
+            },
+            "G64": {
+                "default": 0.020000,
+                "type": "float",
+                "min": 0.0,
+                "max": 0.1,
+                "title": _("G64-Value"),
+                "tooltip": _("value for the G64 command"),
+            },
             "reverse": {
                 "default": False,
                 "type": "bool",
@@ -89,6 +111,16 @@ def setup_defaults(_) -> dict:
                 "title": _("Offset"),
                 "tooltip": _("tool offset"),
             },
+            "objectorder": {
+                "default": "nearest",
+                "type": "select",
+                "options": (
+                    ("nearest", _("nearest")),
+                    ("unordered", _("unordered")),
+                ),
+                "title": _("Object-Order"),
+                "tooltip": _("how order the objects"),
+            },
         },
         "tool": {
             "rate_h": {
@@ -98,6 +130,7 @@ def setup_defaults(_) -> dict:
                 "max": 10000,
                 "title": _("Feed-Rate(Horizontal)"),
                 "tooltip": _("the Horizotal Feetrate"),
+                "unit": "mm/min",
             },
             "rate_v": {
                 "default": 100,
@@ -106,6 +139,7 @@ def setup_defaults(_) -> dict:
                 "max": 10000,
                 "title": _("Feed-Rate(Vertical)"),
                 "tooltip": _("the Vertical Feetrate"),
+                "unit": "mm/min",
             },
             "number": {
                 "default": 1,
@@ -118,10 +152,20 @@ def setup_defaults(_) -> dict:
             "diameter": {
                 "default": 4.0,
                 "type": "float",
+                "step": 0.1,
                 "min": 0.0,
                 "max": 999.0,
                 "title": _("Diameter"),
                 "tooltip": _("setting the Tool-Diameter to calculate the Offsets"),
+                "unit": "LINEARMEASURE",
+            },
+            "blades": {
+                "default": 2,
+                "type": "int",
+                "min": 1,
+                "max": 100,
+                "title": _("Blades"),
+                "tooltip": _("number ob blades"),
             },
             "speed": {
                 "default": 10000,
@@ -130,46 +174,56 @@ def setup_defaults(_) -> dict:
                 "max": 100000,
                 "title": _("Speed"),
                 "tooltip": _("setting the Tool-Speed in RPM"),
+                "unit": "RPM",
+            },
+            "pause": {
+                "default": 1,
+                "type": "int",
+                "min": 0,
+                "max": 60,
+                "title": _("Pause"),
+                "tooltip": _("tool spin up time (G04 Pn)"),
+                "unit": "s",
             },
             "tooltable": {
                 "type": "table",
                 "selectable": True,
                 "default": [
                     {
-                        "name": "Holz-Fräser (klein)",
+                        "name": "Wood cutter (small)",
                         "number": 1,
                         "diameter": 2.5,
                         "lenght": 10.0,
                         "blades": 3,
                     },
                     {
-                        "name": "Holz-Fräser (groß)",
-                        "number": 3,
+                        "name": "Wood cutter (big)",
+                        "number": 2,
                         "diameter": 4.0,
                         "lenght": 12.0,
                         "blades": 2,
                     },
                     {
-                        "name": "Alu-Fräser (groß)",
-                        "number": 4,
+                        "name": "Alu cutter (big)",
+                        "number": 3,
                         "diameter": 6.0,
                         "lenght": 12.0,
                         "blades": 1,
                     },
                     {
                         "name": "Laser",
-                        "number": 1,
+                        "number": 10,
                         "diameter": 0.1,
                         "lenght": 10.0,
                         "blades": 0,
                     },
                 ],
                 "columns": {
-                    "name": "str",
-                    "number": "int",
-                    "diameter": "float",
-                    "lenght": "float",
-                    "blades": "int",
+                    "name": {"type": "str"},
+                    "number": {"type": "int"},
+                    "diameter": {"type": "float"},
+                    "lenght": {"type": "float"},
+                    "blades": {"type": "int"},
                 },
                 "column_defaults": {
                     "name": "",
@@ -186,35 +240,35 @@ def setup_defaults(_) -> dict:
                 "selectable": True,
                 "default": [
                     {
-                        "name": "Aluminium(Langsp.)",
+                        "name": "Aluminum(Long.)",
                         "vc": 200,
                         "fz4": 0.04,
                         "fz8": 0.05,
                         "fz12": 0.10,
                     },
                     {
-                        "name": "Aluminium(Kurzsp.)",
+                        "name": "Aluminum(Short.)",
                         "vc": 150,
                         "fz4": 0.04,
                         "fz8": 0.05,
                         "fz12": 0.10,
                     },
                     {
-                        "name": "NE-Metalle",
+                        "name": "NF-Metals",
                         "vc": 150,
                         "fz4": 0.04,
                         "fz8": 0.05,
                         "fz12": 0.10,
                     },
                     {
-                        "name": "VA-Stahl",
+                        "name": "VA-Steel",
                         "vc": 100,
                         "fz4": 0.05,
                         "fz8": 0.06,
                         "fz12": 0.07,
                     },
                     {
-                        "name": "Duroplaste",
+                        "name": "Thermoset plastic",
                         "vc": 125,
                         "fz4": 0.04,
                         "fz8": 0.08,
@@ -230,7 +284,7 @@ def setup_defaults(_) -> dict:
                     {"name": "GFK", "vc": 125, "fz4": 0.04, "fz8": 0.08, "fz12": 0.10},
                     {"name": "CFK", "vc": 125, "fz4": 0.04, "fz8": 0.08, "fz12": 0.10},
                     {
-                        "name": "Holz",
+                        "name": "Wood",
                         "vc": 2000,
                         "fz4": 0.04,
                         "fz8": 0.08,
@@ -238,11 +292,11 @@ def setup_defaults(_) -> dict:
                     },
                 ],
                 "columns": {
-                    "name": "str",
-                    "vc": "int",
-                    "fz4": "float",
-                    "fz8": "float",
-                    "fz12": "float",
+                    "name": {"type": "str"},
+                    "vc": {"type": "int"},
+                    "fz4": {"type": "float"},
+                    "fz8": {"type": "float"},
+                    "fz12": {"type": "float"},
                 },
                 "column_defaults": {
                     "name": "",
@@ -277,6 +331,7 @@ def setup_defaults(_) -> dict:
                 "max": 100000.0,
                 "title": _("Offset X"),
                 "tooltip": _("Offset X (G54)"),
+                "unit": "LINEARMEASURE",
             },
             "offset_y": {
                 "default": 0.0,
@@ -285,6 +340,7 @@ def setup_defaults(_) -> dict:
                 "max": 100000.0,
                 "title": _("Offset Y"),
                 "tooltip": _("Offset Y (G54)"),
+                "unit": "LINEARMEASURE",
             },
             "offset_z": {
                 "default": 0.0,
@@ -293,6 +349,7 @@ def setup_defaults(_) -> dict:
                 "max": 100000.0,
                 "title": _("Offset Z"),
                 "tooltip": _("Offset Z (G54)"),
+                "unit": "LINEARMEASURE",
             },
         },
         "pockets": {
@@ -303,6 +360,13 @@ def setup_defaults(_) -> dict:
                 "title": _("Pocket"),
                 "tooltip": _("do pocket operation on this object"),
             },
+            "zigzag": {
+                "default": False,
+                "type": "bool",
+                "per_object": True,
+                "title": _("Zickzack"),
+                "tooltip": _("Zickzack"),
+            },
             "islands": {
                 "default": True,
                 "type": "bool",
@@ -334,6 +398,7 @@ def setup_defaults(_) -> dict:
                 "title": _("Width"),
                 "tooltip": _("width of the tabs"),
                 "per_object": True,
+                "unit": "LINEARMEASURE",
             },
             "height": {
                 "default": 2,
@@ -343,6 +408,7 @@ def setup_defaults(_) -> dict:
                 "title": _("Height"),
                 "tooltip": _("height of the tabs"),
                 "per_object": True,
+                "unit": "LINEARMEASURE",
             },
             "type": {
                 "default": "rectangle",
@@ -355,7 +421,51 @@ def setup_defaults(_) -> dict:
                 "tooltip": _("type of the tab"),
             },
         },
-        "maschine": {
+        "leads": {
+            "in": {
+                "default": "off",
+                "type": "select",
+                "options": (
+                    ("off", _("off")),
+                    ("arc", _("arc")),
+                    ("straight", _("straight")),
+                ),
+                "title": _("in-type"),
+                "tooltip": _("type of the lead-in's"),
+            },
+            "in_lenght": {
+                "default": 3.0,
+                "type": "float",
+                "min": 0.1,
+                "max": 100,
+                "title": _("in-lenght"),
+                "tooltip": _("lenght of the lead-in's"),
+                "per_object": True,
+                "unit": "LINEARMEASURE",
+            },
+            "out": {
+                "default": "off",
+                "type": "select",
+                "options": (
+                    ("off", _("off")),
+                    ("arc", _("arc")),
+                    ("straight", _("straight")),
+                ),
+                "title": _("out-type"),
+                "tooltip": _("type of the lead-out's"),
+            },
+            "out_lenght": {
+                "default": 3.0,
+                "type": "float",
+                "min": 0.1,
+                "max": 100,
+                "title": _("out-lenght"),
+                "tooltip": _("lenght of the lead-out's"),
+                "per_object": True,
+                "unit": "LINEARMEASURE",
+            },
+        },
+        "machine": {
             "feedrate": {
                 "default": 1000,
                 "type": "int",
@@ -401,13 +511,13 @@ def setup_defaults(_) -> dict:
                     ("inch", _("inch")),
                 ),
                 "title": _("Unit"),
-                "tooltip": _("Unit of the maschine"),
+                "tooltip": _("Unit of the machine"),
             },
             "g54": {
                 "default": False,
                 "type": "bool",
-                "title": _("maschine supports g54"),
-                "tooltip": _("maschine supports g54"),
+                "title": _("machine supports g54"),
+                "tooltip": _("machine supports g54"),
             },
             "comments": {
                 "default": True,
@@ -415,6 +525,12 @@ def setup_defaults(_) -> dict:
                 "title": _("Comments in output"),
                 "tooltip": _("add comments to output"),
             },
+            "numbers": {
+                "default": False,
+                "type": "bool",
+                "title": _("line numbers"),
+                "tooltip": _("adding line numbers"),
+            },
             "postcommand": {
                 "default": "",
                 "type": "str",
@@ -423,6 +539,12 @@ def setup_defaults(_) -> dict:
             },
         },
         "view": {
+            "autocalc": {
+                "default": True,
+                "type": "bool",
+                "title": _("Auto-Recalculation"),
+                "tooltip": _("update drawing automatically"),
+            },
             "path": {
                 "default": "simple",
                 "type": "select",
@@ -434,6 +556,12 @@ def setup_defaults(_) -> dict:
                 "title": _("Path"),
                 "tooltip": _("how to show the gcode path in the 3d-View"),
             },
+            "colors_show": {
+                "default": True,
+                "type": "bool",
+                "title": _("Colors-Show"),
+                "tooltip": _("showing colors in 3D preview"),
+            },
             "ruler_show": {
                 "default": True,
                 "type": "bool",
@@ -466,11 +594,33 @@ def setup_defaults(_) -> dict:
                 "title": _("Show Object-ID's"),
                 "tooltip": _("shows id of each object"),
             },
+            "arcs": {
+                "default": True,
+                "type": "bool",
+                "title": _("arcs"),
+                "tooltip": _("draw arcs / Interpolation"),
+            },
             "3d_show": {
                 "default": False,
                 "type": "bool",
                 "title": _("Show inputfile in 3d"),
                 "tooltip": _("Show inputfile in 3d if possible"),
             },
+            "color": {
+                "default": (0.5, 0.5, 0.5),
+                "type": "color",
+                "title": _("color"),
+                "tooltip": _("color of the workpeace in 3d view"),
+            },
+            "alpha": {
+                "default": 0.6,
+                "type": "float",
+                "step": 0.1,
+                "decimals": 1,
+                "min": 0,
+                "max": 1.0,
+                "title": _("transparency"),
+                "tooltip": _("transparency of the workpeace in 3d view"),
+            },
         },
     }
